Unit Vector1;
{$IFDEF CPU87} {$N+} {$ELSE} {$N-} {$ENDIF}

INTERFACE

USES Crt,Graph,UNIGRA12;

{$IFNDEF CPU87} TYPE EXTENDED = REAL; {$ENDIF}
CONST
EPS1 = 1.0E-6;
EPS  = 1.0E-12;
TYPE
Line2D = RECORD                                                     
             O: Pt2D;
             U: Vt2D;
           END;

Circle = RECORD                                                      
             M: Pt2D;
             R: EXTENDED;
           END;

(* ----------------------------------------------------------------------- *)
PROCEDURE SquareEquation(A,B,C: EXTENDED; VAR NOS: INTEGER;
                         VAR T1,T2: EXTENDED);
PROCEDURE MakeVt2D(P1,P2: Pt2D; VAR V: Vt2D);
PROCEDURE NormalVector(V: Vt2D; VAR NV : Vt2D);
PROCEDURE LengthVt2D(V: Vt2D; VAR L: EXTENDED);
PROCEDURE ScaleVt2D(V: Vt2D; LG: EXTENDED; VAR ScalV: Vt2D);
PROCEDURE SumVt2D(V1,V2:Vt2D; VAR V: Vt2D);
PROCEDURE DifferenceVt2D(V1,V2: Vt2D; VAR DV: Vt2D);
PROCEDURE MidPoint(P1,P2: Pt2D; VAR M: Pt2D);
PROCEDURE NormDistance(Line: Line2D; P: Pt2D; VAR Dist : EXTENDED);
PROCEDURE Wsymm(V1,V2: Vt2d; P: Pt2D; VAR Wsym: Line2D);
PROCEDURE Ssymm(A,B: Pt2D; VAR Ssym:Line2D);
PROCEDURE CosSinToAngle(C,S: EXTENDED; VAR Ang: EXTENDED);
PROCEDURE AngleBetweenVts2D (V1,V2: Vt2D; VAR Ang:EXTENDED);
PROCEDURE MakeLine2D(P1,P2: Pt2D; VAR Line: Line2D);
PROCEDURE LinearCombinationVt2D(S1,S2: EXTENDED; V1,V2: Vt2D; VAR LC: Vt2D);
PROCEDURE ScalarProductVt2D(V1,V2: Vt2D; VAR SP: EXTENDED);
PROCEDURE TToPLine2D(Ln2D: Line2D; T: EXTENDED; VAR P: Pt2D);
PROCEDURE TtoPCircle(Kreis: Circle; T: EXTENDED; VAR P: Pt2D);
PROCEDURE TtoPHyperbel(MP : Pt2D; A,B,T: EXTENDED; VAR P : Pt2D);
PROCEDURE IntersectionLineCircle(Ln2D: Line2D; Kreis: Circle;
                                 VAR NOS: INTEGER; VAR T1,T2: EXTENDED;
                                 VAR S1,S2: Pt2D);
PROCEDURE IntersectionLines2D(Line1,Line2: Line2D; VAR Intsec: BOOLEAN;
                              VAR T1,T2: EXTENDED; VAR S: Pt2D);
PROCEDURE In_Parameterform(A,B,C: EXTENDED;VAR Line: Line2D);
PROCEDURE Chordale(K1,K2 : Circle; VAR PotLine : Line2D);
PROCEDURE IntersectionCirles2D(K1,K2 : Circle; VAR Intsec : BOOLEAN;
                               VAR S1,S2 : Pt2D);
PROCEDURE In_Normalvektorform(A,B,C: EXTENDED;
                              VAR N: Vt2D; VAR Konst: EXTENDED);
PROCEDURE In_allgemeine_Form(Line:Line2D; VAR A,B,C: EXTENDED);
PROCEDURE NormalVect_toParameterform(N0: Vt2D; Konst: EXTENDED;
                                     VAR Line: Line2D);
PROCEDURE Spiegeln(G: Line2D; P : Pt2D; VAR PS : Pt2D);
PROCEDURE NormLinie;
PROCEDURE Hilfslinie1;
PROCEDURE HilfsLinie2;
PROCEDURE WeitStrichliert;
PROCEDURE EngStrichliert;
PROCEDURE StrichPunktiert;                   (* als Symmetrieachse         *)
PROCEDURE BreitLinie;

PROCEDURE Marker(P:Pt2D);            (*   Plotter fehlt !!!!!!!!!! *)

(* ----------------------------------------------------------------------- *)
IMPLEMENTATION

PROCEDURE SquareEquation(A,B,C: EXTENDED; VAR NOS: INTEGER;
                         VAR T1,T2: EXTENDED);
CONST
Eps1 = 1.0E-6;             (* muss groesser als allgemeinenes Epsilon sein *)
                           (* bestimmt die Tangentengenauigkeit            *)
VAR
D,TC: EXTENDED;
BEGIN
  CASE Null(A,Eps1) OF
    TRUE: BEGIN
      IF Null(B,Eps1) THEN NOS:=0
      ELSE BEGIN NOS:=1; T1:=-C/B; T2:=T1; END;
    END;
    FALSE: BEGIN
      D:=B*B-4*A*C;
      IF Null(D,Eps1) THEN BEGIN
        NOS:=1 ;
        T1:=-B/2/A; T2:=T1;
      END
      ELSE BEGIN
        IF D>0 THEN BEGIN
          NOS:=2;
          T1:=(-B+SQRT(D))/2/A;
          T2:=(-B-SQRT(D))/2/A;
          IF T1>T2 THEN BEGIN
            TC:=T1; T1:=T2; T2:=TC;
          END;
        END
        ELSE NOS:=0;
      END;
    END;
  END;
END;

PROCEDURE MakeVt2d(P1,P2: Pt2D; VAR V: Vt2D);
  BEGIN
   V.X := P2.X - P1.X;
   V.Y := P2.Y - P1.Y;
  END;

PROCEDURE NormalVector(V: Vt2D; VAR NV : Vt2D);
  BEGIN
    NV.X := -V.Y;
    NV.Y :=  V.X;
  END;

PROCEDURE LengthVt2D(V: Vt2D; VAR L: EXTENDED);
BEGIN
  L := SQRT(SQR(V.X)+SQR(V.Y)+Eps);
END;

PROCEDURE ScaleVt2D(V: Vt2D; LG: EXTENDED; VAR ScalV: Vt2D);
VAR L: EXTENDED;
BEGIN
  ScalV:=V;
  LengthVt2D(V, L);
  IF NOT Null(L,Eps) THEN BEGIN
    ScalV.X:=V.X/L*LG;
    ScalV.Y:=V.Y/L*LG;
  END;
END;

PROCEDURE SumVt2d(V1,V2:Vt2D; VAR V: Vt2D
);
  BEGIN
   V.X := V1.X + V2.X;
   V.Y := V1.Y + V2.Y;
  END;

PROCEDURE DifferenceVt2D(V1,V2: Vt2D; VAR DV: Vt2D);
BEGIN
  DV.X:=V1.X-V2.X;
  DV.Y:=V1.Y-V2.Y;
END;

PROCEDURE MidPoint(P1,P2: Pt2D; VAR M: Pt2D);
   BEGIN
     M.X := (P1.X+P2.X)/2.0;
     M.Y := (P1.Y+P2.Y)/2.0;
   END;

PROCEDURE NormDistance(Line: Line2D; P: Pt2D; VAR Dist : EXTENDED);
  VAR
  H,N0 : Vt2D;
  D : EXTENDED;
  BEGIN
    H.X := -Line.U.Y; H.Y := Line.U.X;        (* vorlaeufiger Normalvektor *)
    IF(Line.O.X*H.X + Line.O.Y*H.Y) < 0.0 THEN BEGIN
      H.X := -H.X;                            (* Orientierung aendern, so  *)
      H.Y := -H.Y;                            (* dass Konst immer positiv  *)
      END;
    ScaleVt2D(H,1.0,N0);                        (*  sollte schon 1 sein !! *)
    D := N0.X*Line.O.X + N0.Y*Line.O.Y;
    Dist := N0.X*P.X + N0.Y*P.Y-D;
  END;

PROCEDURE Wsymm(V1,V2: Vt2d; P: Pt2D; VAR Wsym: Line2D);
VAR
V,V0: Vt2d;
  BEGIN
    SumVt2d(V1,V2,V);
    ScaleVt2D(V,1.0,V0);
    Wsym.U := V0;
    Wsym.O :=  P;
  END;

PROCEDURE Ssymm(A,B: Pt2D; VAR Ssym:Line2D);
  VAR
  M : Pt2D;
  Vh,Nh : Vt2D;
  BEGIN
    MidPoint(A,B,M);
    MakeVt2D(A,B,Vh);
    NormalVector(Vh,Nh);
    ScaleVt2D(Nh,1.0,Nh);
    Ssym.O := M; Ssym.U := Nh;
  END;

PROCEDURE CosSinToAngle(C,S: EXTENDED; VAR Ang: EXTENDED);
BEGIN
   IF Null(C,Eps) AND (S<0.0) THEN Ang := -PI/2.0;
   IF Null(C,Eps) AND (S>=0.0) THEN Ang := PI/2.0;
   IF NOT Null(C,Eps) THEN Ang := ARCTAN(S/C);
   IF (C<0) AND (S<0.0) THEN Ang := Ang-PI;
   IF (C<0) AND (S>=0.0) THEN Ang := Ang+PI;
END;

PROCEDURE AngleBetweenVts2D (V1,V2: Vt2D; VAR Ang:EXTENDED);
VAR C,S: EXTENDED;
BEGIN
   C:=V1.X*V2.X + V1.Y*V2.Y;
   S:=V1.X*V2.Y - V1.Y*V2.X;
   CosSinToAngle(C,S, Ang);
END;

PROCEDURE MakeLine2D(P1,P2: Pt2D; VAR Line: Line2D);
  VAR
  hv : Vt2D;
  BEGIN
    MakeVt2D(P1,P2,hv);
    ScaleVt2D(hv,1.0,hv);
    Line.O := P1;
    Line.U := hv;
  END;

PROCEDURE LinearCombinationVt2D(S1,S2: EXTENDED; V1,V2: Vt2D; VAR LC: Vt2D);
BEGIN
  LC.X:=S1*V1.X+S2*V2.X;
  LC.Y:=S1*V1.Y+S2*V2.Y;
END;

PROCEDURE ScalarProductVt2D(V1,V2: Vt2D; VAR SP: EXTENDED);
BEGIN
  SP:=V1.X*V2.X+V1.Y*V2.Y;
END;

PROCEDURE TToPLine2D(Ln2D: Line2D; T: EXTENDED; VAR P: Pt2D);
BEGIN
  LinearCombinationVt2D(1,T,Ln2D.O,Ln2D.U, P);
END;

PROCEDURE TtoPCircle(Kreis: Circle; T: EXTENDED; VAR P: Pt2D);
  BEGIN
    P.X := Kreis.M.X+Kreis.R*COS(T);
    P.Y := Kreis.M.Y+Kreis.R*SIN(T);
  END;

PROCEDURE TtoPHyperbel(MP : Pt2D; A,B,T: EXTENDED;
                       VAR P : Pt2D);
  BEGIN
    IF NOT Null(COS(T),Eps)THEN BEGIN
      P.X := MP.X + A/COS(T);
      P.Y := MP.Y + B*(SIN(T)/COS(T));
    END;
  END;

PROCEDURE IntersectionLineCircle(Ln2D: Line2D; Kreis: Circle;
                                 VAR NOS: INTEGER; VAR T1,T2: EXTENDED;
                                 VAR S1,S2: Pt2D);
VAR MO2,MOU,A,B,C: EXTENDED;
    MO: Vt2D;
BEGIN
  DifferenceVt2D(Ln2D.O,Kreis.M, MO);
  ScalarProductVt2D(MO,MO, MO2);
  ScalarProductVt2D(MO,Ln2D.U, MOU);
  A := 1; B := 2*MOU; C := MO2-SQR(Kreis.R);
  SquareEquation(A,B,C, NOS,T1,T2);
  IF NOS>0 THEN BEGIN
    TToPLine2D(Ln2D,T1, S1);
    TToPLine2D(Ln2D,T2, S2);
  END;
END;

PROCEDURE IntersectionLines2D(Line1,Line2: Line2D; VAR Intsec: BOOLEAN;
                              VAR T1,T2: EXTENDED; VAR S: Pt2D);
VAR Num,Den: EXTENDED;
    N2,DV: Vt2D;
    V: Vt2D;
BEGIN
  Intsec := FALSE;
  N2.X := Line2.U.Y; N2.Y := -Line2.U.X;
  DifferenceVt2D(Line1.O, Line2.O, DV);
  ScalarProductVt2D(DV,N2, Num);
  ScalarProductVt2D(Line1.U,N2, Den);
  IF NOT Null(Den,Eps) THEN BEGIN
    T1 := -Num/Den;
    TToPLine2D(Line1,T1, S);
    DifferenceVt2D(S,Line2.O, V);       (* nicht mehr notwendig, nur mehr *)
    ScalarProductVt2D(V,Line2.U, T2);   (* zur Berechnung von t2          *)
    Intsec := TRUE;
  END;
END;

PROCEDURE In_Parameterform(A,B,C: EXTENDED;VAR Line: Line2D);
  BEGIN
    Line.U.X :=  B;                               (* Richtungsvektor       *)
    Line.U.Y := -A;                               (* Orientierung beliebig *)
                                                  (* Line.U.X :=  -B;      *)
                                                  (* Line.U.Y :=   A;      *)
                                                  (* auch moeglich !!!     *)
                                                  (* Richtungsvektor       *)
    ScaleVt2D(Line.U,1,Line.U);                   (* auf Einheitsvektor    *)
                                                  (* fixen Punkt bestimmen *)
    IF Null(A,Eps) THEN BEGIN                     (* Parallele zur X-Achse *)
      Line.O.Y := C/B;
      Line.O.X := 0.0;                                (* beliebig waehlbar *)
    END;
    IF Null(B,Eps) THEN BEGIN                     (* Parallele zur Y-Achse *)
      Line.O.X := C/A;
      Line.O.Y := 0.0;                                (* beliebig waehlbar *)
    END;
    IF ((NOT Null(A,Eps)) AND (NOT Null(B,Eps))) THEN BEGIN
      Line.O.X :=  0.0;                               (* beliebig waehlbar *)
      Line.O.Y :=  C/B;
    END;
  END;

PROCEDURE Chordale(K1,K2 : Circle; VAR PotLine : Line2D);
   VAR
   A,B,C : EXTENDED;                          (* Koeffizienten der Gerade *)
   M1X,M1Y,R1,M2X,M2Y,R2 : EXTENDED;          (* nur zur Sicherheit !     *)
   PotH : Line2D;
   BEGIN
     M1X := K1.M.X; M1Y := K1.M.Y; R1 := K1.R;
     M2X := K2.M.X; M2Y := K2.M.Y; R2 := K2.R;
     A := 2.0*M2X - 2.0*M1X;
     B := 2.0*M2Y - 2.0*M1Y;
     C := M2X*M2X - M1X*M1X + M2Y*M2Y - M1Y*M1Y + R1*R1 - R2*R2;
     In_Parameterform(A,B,C,PotH);
     PotLine := PotH;
   END;

PROCEDURE IntersectionCirles2D(K1,K2 : Circle; VAR Intsec : BOOLEAN;
                               VAR S1,S2 : Pt2D);
  VAR
  Nos : INTEGER;
  T1,T2 : EXTENDED;
  S1H,S2H : Pt2D;
  PotH : Line2D;
  BEGIN
    Intsec := FALSE;
    Chordale(K1,K2,PotH);
    IntersectionLineCircle(PotH,K1,NoS,T1,T2,S1H,S2H);              
    IF Nos > 0 THEN Intsec := TRUE;
    S1 := S1H; S2 := S2H;
  END;

PROCEDURE In_Normalvektorform(A,B,C: EXTENDED;
                              VAR N: Vt2D; VAR Konst: EXTENDED);
  VAR
  H: Pt2D;

  BEGIN
    IF C < 0 THEN BEGIN
      A := -A; B := -B; C := -C;
    END;                                              (* Gleichung  mal -1 *)
    N.X :=  A;                                          (* Normalvektor    *)
    N.Y :=  B;
                                                  (* Normalvektor          *)
    ScaleVt2D(N,1,N);                             (* auf Einheitsvektor    *)
                                                  (* jetzt einen fixen     *)     
                                                  (* Gerad.Punkt bestimmen *)
                                                  (* = H wie Hilfspunkt    *)
    IF Null(A,Eps) THEN BEGIN                     (* Parallele zur X-Achse *)
      H.Y := C/B;
      H.X := 0.0;                                     (* beliebig waehlbar *)
    END;
    IF Null(B,Eps) THEN BEGIN                     (* Parallele zur Y-Achse *)
      H.X := C/A;
      H.Y := 0.0;                                     (* beliebig waehlbar *)
    END;
    IF ((NOT Null(A,Eps)) AND (NOT Null(B,Eps))) THEN BEGIN
      H.X :=  0.0;                                    (* beliebig waehlbar *)
      H.Y :=  C/B;
    END;
    ScalarProductVt2D(N,H,Konst);
  END;

PROCEDURE In_allgemeine_Form(Line:Line2D; VAR A,B,C: EXTENDED);
  VAR
  H: Vt2D;                   (*  Hilfsvektor als vorlauefiger Normalvektor *)
  N: Vt2D;
  BEGIN
    H.X := -Line.U.Y; H.Y := Line.U.X;        (* vorlaeufiger Normalvektor *)
    IF(Line.O.X*H.X + Line.O.Y*H.Y) < 0.0 THEN BEGIN
      N.X := -H.X;                            (* Orientierung aendern, so  *)
      N.Y := -H.Y;                            (* dass Konst immer positiv  *)
      END ELSE BEGIN
        N := H;                               (* N=richtiger Normalvektor  *)
    END;
    A := N.X; B := N.Y;
    C := Line.O.X*N.X + Line.O.Y*N.Y;        (* Abstand vom Ursprung, weil *)
  END;                                       (* N = Einheitsvektor         *)

PROCEDURE NormalVect_toParameterform(N0: Vt2D; Konst: EXTENDED;
                                     VAR Line: Line2D);
  VAR
  OA: Pt2D;                                       (* zum Zwischenspeichern *) 
  R: Vt2D;
  BEGIN                                               
    IF NOT(NULL(N0.X,EPS)) THEN BEGIN                 (* fixer Punkt       *)
      OA.X := Konst/N0.X;
      IF (NULL(N0.Y,EPS)) THEN OA.Y := 0.0 ELSE OA.Y := Konst/N0.Y;
    END ELSE BEGIN
       OA.Y := Konst/N0.Y;
       IF (NULL(N0.X,EPS)) THEN OA.X := 0.0 ELSE OA.X := Konst/N0.X;
    END;
    R.X := -N0.Y; R.Y := N0.X;                         (* Richtungsvektor  *)
    Line.O := OA; Line.U := R;                         (* in Typ speichern *)
  END;

PROCEDURE Spiegeln(G: Line2D; P : Pt2D; VAR PS : Pt2D);
  VAR
  N : Vt2D;
  Gh : Line2D;
  Sg : Line2D;
  Intsec: BOOLEAN;
  T1,T2 : EXTENDED;
  PSh,S     : Pt2D;
  BEGIN
    Gh := G;
    Sg.O := P;
    N.X := -G.U.Y;
    N.Y :=  G.U.X;
    Sg.U := N;
    IntersectionLines2D(Sg,Gh,Intsec,T1,T2,S);
    TToPLine2D(Sg,2.0*T1,PSh);
    PS := PSh;
  END;

PROCEDURE Marker(P:Pt2D);
   VAR
   X,Y : INTEGER;
   BEGIN
     MovetoPt2D(P);
     X := GetX;  Y := GetY;
     Line(X-2,Y,X+2,Y);
     Line(X,Y-2,X,Y+2);           (* Plotter fehlt !!!!!!!! *)
   END;

PROCEDURE NormLinie;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0.15');          (* 0.15 mm                    *)
               WRITE(AUX,'UL');              (* 6.:                        *)
               WRITE(AUX,'LT');              (* solid                      *)
    END;
  SetLineStyle(0,0,1);
  END;

PROCEDURE Hilfslinie1;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');              (* ganz schwache Hilfslinie: *)
               WRITE(AUX,'UL8,1,9,1,9,1,9');  (* 1.:                       *)
               WRITE(AUX,'LT8,4.0,1');       
    END;
    SetLineStyle(4,$8080,1);
  END;

PROCEDURE HilfsLinie2;
  BEGIN
    IF PLOT THEN BEGIN
          {    WRITE(AUX,'PW0.3');       }    (* staerkere Hilfslinie      *)
          {    WRITE(AUX,'LT1,0.8,1');   }    (* hellgrau length 1.0 mm    *)
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');
               WRITE(AUX,'UL8,2,7,2,7,2,7');
               WRITE(AUX,'LT8,2.0,1');      
    END;
    SetLineStyle(1,0,1);
  END;

PROCEDURE WeitStrichliert;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');             (* duennste Linie strichliert *)
               WRITE(AUX,'UL8,7,4,7,4,7,4'); (* 3.:                        *)
               WRITE(AUX,'LT8,7.0,1');       (*  weite Abstaende           *)
    END;
    SetLineStyle(4,$FF00,1);
  END;

PROCEDURE EngStrichliert;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');             (* duennste Linie strichliert *)
               WRITE(AUX,'UL8,7,2,7,2,7,2'); (* 5.:                        *)
               WRITE(AUX,'LT8,10.0,1');
    END;
    SetLineStyle(3,0,1);
  END;

PROCEDURE StrichPunktiert;                   (* als Symmetrieachse         *)
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0');             (* duennste Linie             *)
               WRITE(AUX,'UL8,8,2,2,2,8,2,2,2,8,2,2,2');
               WRITE(AUX,'LT8,15.0,1');      (* strichpunktiert mit klein. *)
    END;                                     (* Strich als Punkt           *)
    SetLineStyle(4,$F18F,1);                  
  END;                                           

PROCEDURE BreitLinie;
  BEGIN
    IF PLOT THEN BEGIN
               WRITE(AUX,'WU0');
               WRITE(AUX,'PW0.3');           (* 0.3 durchgehend            *)
               WRITE(AUX,'UL');              (* 7.:                        *)
               WRITE(AUX,'LT');
    END;
    SetLineStyle(0,0,1);
  END;
   


(* ------------------------- Initialisierung ----------------------------- *)
BEGIN
END.